@@ -3,11 +3,12 @@
  * This program is distributed under the GNU General Public License, version 2.
  * A copy of this license is included with this source.
  *
- * Copyright 2000-2004, Jack Moffitt <jack@xiph.org, 
+ * Copyright 2000-2004, Jack Moffitt <jack@xiph.org,
  *                      Michael Smith <msmith@xiph.org>,
  *                      oddsock <oddsock@xiph.org>,
  *                      Karl Heyes <karl@xiph.org>
  *                      and others (see AUTHORS for details).
+ * Copyright 2014-2018, Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>,
  */
 
 /* format_ogg.c
@@ -26,11 +27,18 @@
 
 #include <ogg/ogg.h>
 
+#include <igloo/sp.h>
+#include <igloo/error.h>
+
+#include "global.h"
 #include "refbuf.h"
 #include "source.h"
 #include "client.h"
 
 #include "stats.h"
+#include "playlist.h"
+#include "event_stream.h"
+#include "event.h"
 #include "format.h"
 #include "format_ogg.h"
 #include "format_vorbis.h"
@@ -55,13 +63,14 @@
 
 struct _ogg_state_tag;
 
-static void format_ogg_free_plugin (format_plugin_t *plugin);
-static int  create_ogg_client_data(source_t *source, client_t *client);
-static void free_ogg_client_data (client_t *client);
+static void format_ogg_free_plugin(format_plugin_t *plugin);
+static int create_ogg_client_data(source_t *source, client_t *client);
+static void free_ogg_client_data(client_t *client);
 
-static void write_ogg_to_file (struct source_tag *source, refbuf_t *refbuf);
-static refbuf_t *ogg_get_buffer (source_t *source);
-static int write_buf_to_client (client_t *client);
+static void write_ogg_to_file(source_t *source, refbuf_t *refbuf);
+static void on_file_close(source_t *source);
+static refbuf_t *ogg_get_buffer(source_t *source);
+static int write_buf_to_client(client_t *client);
 
 
 struct ogg_client
@@ -110,7 +119,7 @@ void format_ogg_attach_header (ogg_state_t *ogg_info, ogg_page *page)
 }
 
 
-void format_ogg_free_headers (ogg_state_t *ogg_info)
+void format_ogg_free_headers(ogg_state_t *ogg_info)
 {
     refbuf_t *header;
 
@@ -121,7 +130,7 @@ void format_ogg_free_headers (ogg_state_t *ogg_info)
     {
         refbuf_t *to_release = header;
         header = header->next;
-        refbuf_release (to_release);
+        refbuf_release(to_release);
     }
     ogg_info->header_pages = NULL;
     ogg_info->header_pages_tail = NULL;
@@ -130,14 +139,14 @@ void format_ogg_free_headers (ogg_state_t *ogg_info)
 
 
 /* release the memory used for the codec and header pages from the module */
-static void free_ogg_codecs (ogg_state_t *ogg_info)
+static void free_ogg_codecs(ogg_state_t *ogg_info)
 {
     ogg_codec_t *codec;
 
     if (ogg_info == NULL)
         return;
 
-    format_ogg_free_headers (ogg_info);
+    format_ogg_free_headers(ogg_info);
 
     /* now free the codecs */
     codec = ogg_info->codecs;
@@ -146,8 +155,8 @@ static void free_ogg_codecs (ogg_state_t *ogg_info)
     {
         ogg_codec_t *next = codec->next;
         if (codec->possible_start)
-            refbuf_release (codec->possible_start);
-        codec->codec_free (ogg_info, codec);
+            refbuf_release(codec->possible_start);
+        codec->codec_free(ogg_info, codec);
         codec = next;
     }
     ogg_info->codecs = NULL;
@@ -157,25 +166,30 @@ static void free_ogg_codecs (ogg_state_t *ogg_info)
 }
 
 
-int format_ogg_get_plugin (source_t *source)
+int format_ogg_get_plugin(source_t *source)
 {
     format_plugin_t *plugin;
-    ogg_state_t *state = calloc (1, sizeof (ogg_state_t));
+    ogg_state_t *state = calloc(1, sizeof(ogg_state_t));
 
-    plugin = (format_plugin_t *)calloc(1, sizeof(format_plugin_t));
+    plugin = (format_plugin_t *) calloc(1, sizeof(format_plugin_t));
 
     plugin->type = FORMAT_TYPE_OGG;
     plugin->get_buffer = ogg_get_buffer;
     plugin->write_buf_to_client = write_buf_to_client;
     plugin->write_buf_to_file = write_ogg_to_file;
+    plugin->on_file_close = on_file_close;
     plugin->create_client_data = create_ogg_client_data;
     plugin->free_plugin = format_ogg_free_plugin;
     plugin->set_tag = NULL;
     if (strcmp (httpp_getvar (source->parser, "content-type"), "application/x-ogg") == 0)
         httpp_setvar (source->parser, "content-type", "application/ogg");
-    plugin->contenttype = httpp_getvar (source->parser, "content-type");
+
+    if (igloo_sp_replace(httpp_getvar(source->parser, "content-type"), &(plugin->contenttype), igloo_instance) != igloo_ERROR_NONE) {
+        ICECAST_LOG_ERROR("Cannot set content type for Ogg source %#H. BAD.", source->mount);
+    }
 
     ogg_sync_init (&state->oy);
+    vorbis_comment_init(&plugin->vc);
 
     plugin->_state = state;
     source->format = plugin;
@@ -192,12 +206,13 @@ static void format_ogg_free_plugin (format_plugin_t *plugin)
 
     /* free memory associated with this plugin instance */
     free_ogg_codecs (state);
-    free (state->artist);
-    free (state->title);
 
     ogg_sync_clear (&state->oy);
+
     free (state);
 
+    vorbis_comment_clear(&plugin->vc);
+
     free (plugin);
 }
 
@@ -213,7 +228,7 @@ static int process_initial_page (format_plugin_t *plugin, ogg_page *page)
         ogg_info->bitrate = 0;
         ogg_info->codec_sync = NULL;
         /* need to zap old list of codecs when next group of BOS pages appear */
-        free_ogg_codecs (ogg_info);
+        free_ogg_codecs(ogg_info);
     }
     do
     {
@@ -223,7 +238,7 @@ static int process_initial_page (format_plugin_t *plugin, ogg_page *page)
             ogg_info->error = 1;
             return -1;
         }
-        codec = initial_vorbis_page (plugin, page);
+        codec = initial_vorbis_page(plugin, page);
         if (codec)
             break;
 #ifdef HAVE_THEORA
@@ -231,10 +246,10 @@ static int process_initial_page (format_plugin_t *plugin, ogg_page *page)
         if (codec)
             break;
 #endif
-        codec = initial_midi_page (plugin, page);
+        codec = initial_midi_page(plugin, page);
         if (codec)
             break;
-        codec = initial_flac_page (plugin, page);
+        codec = initial_flac_page(plugin, page);
         if (codec)
             break;
 #ifdef HAVE_SPEEX
@@ -242,13 +257,13 @@ static int process_initial_page (format_plugin_t *plugin, ogg_page *page)
         if (codec)
             break;
 #endif
-        codec = initial_kate_page (plugin, page);
+        codec = initial_kate_page(plugin, page);
         if (codec)
             break;
-        codec = initial_skeleton_page (plugin, page);
+        codec = initial_skeleton_page(plugin, page);
         if (codec)
             break;
-        codec = initial_opus_page (plugin, page);
+        codec = initial_opus_page(plugin, page);
         if (codec)
             break;
 
@@ -274,17 +289,17 @@ static int process_initial_page (format_plugin_t *plugin, ogg_page *page)
  * artist and title are provided separately so here we update the stats
  * and write log entry if required.
  */
-static void update_comments (source_t *source)
+static void update_comments(source_t *source)
 {
     ogg_state_t *ogg_info = source->format->_state;
-    char *title = ogg_info->title;
-    char *artist = ogg_info->artist;
+    char *title = vorbis_comment_query(&source->format->vc, "TITLE", 0);
+    char *artist = vorbis_comment_query(&source->format->vc, "ARTIST", 0);
     char *metadata = NULL;
     unsigned int len = 1; /* space for the nul byte at least */
     ogg_codec_t *codec;
     char codec_names [100] = "";
 
-    if (ogg_info->artist)
+    if (artist)
     {
         if (title)
         {
@@ -315,6 +330,10 @@ static void update_comments (source_t *source)
     }
     stats_event (source->mount, "artist", artist);
     stats_event (source->mount, "title", title);
+    stats_event (source->mount, "display-title", title);
+
+    playlist_push_track(source->history, &source->format->vc);
+    event_stream_emit_vc(source, &source->format->vc);
 
     codec = ogg_info->codecs;
     while (codec)
@@ -354,6 +373,7 @@ static refbuf_t *complete_buffer (source_t *source, refbuf_t *refbuf)
     if (ogg_info->log_metadata)
     {
         update_comments (source);
+        event_emit_va("format-metadata-changed", EVENT_EXTRA_SOURCE, source, EVENT_EXTRA_LIST_END);
         ogg_info->log_metadata = 0;
     }
     /* listeners can start anywhere unless the codecs themselves are
@@ -367,7 +387,7 @@ static refbuf_t *complete_buffer (source_t *source, refbuf_t *refbuf)
 /* process the incoming page. this requires searching through the
  * currently known codecs that have been seen in the stream
  */
-static refbuf_t *process_ogg_page (ogg_state_t *ogg_info, ogg_page *page)
+static refbuf_t *process_ogg_page(ogg_state_t *ogg_info, ogg_page *page, format_plugin_t *plugin)
 {
     ogg_codec_t *codec = ogg_info->codecs;
     refbuf_t *refbuf = NULL;
@@ -377,7 +397,7 @@ static refbuf_t *process_ogg_page (ogg_state_t *ogg_info, ogg_page *page)
         if (ogg_page_serialno (page) == codec->os.serialno)
         {
             if (codec->process_page)
-                refbuf = codec->process_page (ogg_info, codec, page);
+                refbuf = codec->process_page(ogg_info, codec, page, plugin);
             break;
         }
 
@@ -392,12 +412,12 @@ static refbuf_t *process_ogg_page (ogg_state_t *ogg_info, ogg_page *page)
  * just add an incoming page to the codecs and process it until either
  * more data is needed or we prodice a buffer for the queue.
  */
-static refbuf_t *ogg_get_buffer (source_t *source)
+static refbuf_t *ogg_get_buffer(source_t *source)
 {
     ogg_state_t *ogg_info = source->format->_state;
     format_plugin_t *format = source->format;
     char *data = NULL;
-    int bytes = 0;
+    ssize_t bytes = 0;
 
     while (1)
     {
@@ -410,7 +430,7 @@ static refbuf_t *ogg_get_buffer (source_t *source)
             /* if a codec has just been given a page then process it */
             if (codec && codec->process)
             {
-                refbuf = codec->process (ogg_info, codec);
+                refbuf = codec->process (ogg_info, codec, source->format);
                 if (refbuf)
                     return complete_buffer (source, refbuf);
 
@@ -426,7 +446,7 @@ static refbuf_t *ogg_get_buffer (source_t *source)
                 else
                 {
                     ogg_info->bos_completed = 1;
-                    refbuf = process_ogg_page (ogg_info, &page);
+                    refbuf = process_ogg_page (ogg_info, &page, source->format);
                 }
                 if (ogg_info->error)
                 {
@@ -443,8 +463,10 @@ static refbuf_t *ogg_get_buffer (source_t *source)
         }
         /* we need more data to continue getting pages */
         data = ogg_sync_buffer (&ogg_info->oy, 4096);
+        if (!data)
+            return NULL;
 
-        bytes = client_read_bytes (source->client, data, 4096);
+        bytes = client_body_read(source->client, data, 4096);
         if (bytes <= 0)
         {
             ogg_sync_wrote (&ogg_info->oy, 0);
@@ -456,7 +478,7 @@ static refbuf_t *ogg_get_buffer (source_t *source)
 }
 
 
-static int create_ogg_client_data (source_t *source, client_t *client) 
+static int create_ogg_client_data (source_t *source, client_t *client)
 {
     struct ogg_client *client_data = calloc (1, sizeof (struct ogg_client));
     int ret = -1;
@@ -501,10 +523,10 @@ static int send_ogg_headers (client_t *client, refbuf_t *headers)
         unsigned len = refbuf->len - client_data->pos;
         int ret;
 
-        ret = client_send_bytes (client, data, len);
+        ret = client_send_bytes(client, data, len);
         if (ret > 0)
-           written += ret;
-        if (ret < (int)len)
+            written += ret;
+        if (ret < (int) len)
             return written ? written : -1;
         client_data->pos += ret;
         if (client_data->pos == refbuf->len)
@@ -523,7 +545,7 @@ static int send_ogg_headers (client_t *client, refbuf_t *headers)
 /* main client write routine for sending ogg data. Each refbuf has a
  * single page so we only need to determine if there are new headers
  */
-static int write_buf_to_client (client_t *client)
+static int write_buf_to_client(client_t *client)
 {
     refbuf_t *refbuf = client->refbuf;
     char *buf = refbuf->data + client->pos;
@@ -558,22 +580,7 @@ static int write_buf_to_client (client_t *client)
 }
 
 
-static int write_ogg_data (struct source_tag *source, refbuf_t *refbuf)
-{
-    int ret = 1;
-
-    if (fwrite (refbuf->data, 1, refbuf->len, source->dumpfile) != refbuf->len)
-    {
-        ICECAST_LOG_WARN("Write to dump file failed, disabling");
-        fclose (source->dumpfile);
-        source->dumpfile = NULL;
-        ret = 0;
-    }
-    return ret;
-}
-
-
-static void write_ogg_to_file (struct source_tag *source, refbuf_t *refbuf)
+static void write_ogg_to_file (source_t *source, refbuf_t *refbuf)
 {
     ogg_state_t *ogg_info = source->format->_state;
 
@@ -582,13 +589,17 @@ static void write_ogg_to_file (struct source_tag *source, refbuf_t *refbuf)
         refbuf_t *header = refbuf->associated;
         while (header)
         {
-            if (write_ogg_data (source, header) == 0)
+            if (!source_write_dumpfile(source, header->data, header->len))
                 return;
             header = header->next;
         }
         ogg_info->file_headers = refbuf->associated;
     }
-    write_ogg_data (source, refbuf);
+    source_write_dumpfile(source, refbuf->data, refbuf->len);
 }
 
-
+static void on_file_close(source_t *source)
+{
+    ogg_state_t *ogg_info = source->format->_state;
+    ogg_info->file_headers = NULL;
+}
